require './spec/spec_helper'

module MysqlSlaver
  describe Executor do
    let(:ssh_port) { nil }
    let(:params)   { {ssh_port: ssh_port} }

    subject(:executor) { described_class.new(params) }

    before do
      allow(executor).to receive(:log)
    end

    describe "#ssh_command" do
      it "formats command" do
        expect(executor.ssh_command("foo", "myhost")).to eq("ssh myhost 'foo'")
      end

      context "with non-standard ssh port" do
        let(:ssh_port) { 64389 }

        it "formats command" do
          expect(executor.ssh_command("foo", "myhost")).to eq("ssh -p 64389 myhost 'foo'")
        end
      end
    end

    describe "#execute" do
      let(:succeeded) { true }

      before do
        allow(executor).to receive(:success?).and_return(succeeded)
      end

      context "when command succeeds" do
        let(:succeeded) { true }

        it "runs command in backticks" do
          expect(executor).to receive(:`).with('ls')
          executor.execute('ls')
        end

        it "returns the command output" do
          expect(executor).to receive(:`).with('ls').and_return("foo\nbar\nbaz")
          expect(executor.execute('ls')).to eq("foo\nbar\nbaz")
        end
      end

      context "when command fails" do
        let(:succeeded) { false }

        before do
          allow(executor).to receive(:`)
        end

        it "runs command in backticks" do
          expect(executor).to receive(:`).with('ls')
          executor.execute('ls')
        end

        it "returns nil" do
          expect(executor.execute('ls')).to be_nil
        end
      end

      context "dry-run" do
        let(:params) { super().merge(dry_run: true) }

        it "doesn't run the command" do
          expect(executor).to_not receive(:`)
          executor.execute('ls')
        end

        it "returns dummy result" do
          expect(executor.execute('ls')).to eq("[DUMMY RESULT]")
        end
      end
    end
  end
end
